@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /*
@@ -41,6 +41,7 @@
 #include "gtkutils.h"
 #include "stock_pixmap.h"
 #include "prefs_common.h"
+#include "prefs_gtk.h"
 #include "addressadd.h"
 #include "addritem.h"
 #include "addrbook.h"
@@ -57,14 +58,13 @@ typedef struct {
 
 static struct _AddressAdd_dlg {
 	GtkWidget *window;
+	GtkWidget *picture;
 	GtkWidget *entry_name;
 	GtkWidget *label_address;
 	GtkWidget *entry_remarks;
 	GtkWidget *tree_folder;
 	GtkWidget *ok_btn;
 	GtkWidget *cancel_btn;
-	GtkWidget *statusbar;
-	gint status_cid;
 	FolderInfo *fiSelected;
 } addressadd_dlg;
 
@@ -89,18 +89,6 @@ static void addressadd_free_folderinfo( FolderInfo *fi ) {
 	g_free( fi );
 }
 
-/*
-* Edit functions.
-*/
-static void addressadd_status_show( gchar *msg ) {
-	if( addressadd_dlg.statusbar != NULL ) {
-		gtk_statusbar_pop( GTK_STATUSBAR(addressadd_dlg.statusbar), addressadd_dlg.status_cid );
-		if( msg ) {
-			gtk_statusbar_push( GTK_STATUSBAR(addressadd_dlg.statusbar), addressadd_dlg.status_cid, msg );
-		}
-	}
-}
-
 static gint addressadd_delete_event( GtkWidget *widget, GdkEventAny *event, gboolean *cancelled ) {
 	addressadd_cancelled = TRUE;
 	gtk_main_quit();
@@ -155,9 +143,11 @@ static void addressadd_size_allocate_cb(GtkWidget *widget,
 
 static void addressadd_create( void ) {
 	GtkWidget *window;
-	GtkWidget *vbox;
+	GtkWidget *vbox, *hbox;
+	GtkWidget *frame;
 	GtkWidget *table;
 	GtkWidget *label;
+	GtkWidget *picture;
 	GtkWidget *entry_name;
 	GtkWidget *label_addr;
 	GtkWidget *entry_rems;
@@ -167,13 +157,11 @@ static void addressadd_create( void ) {
 	GtkWidget *hbbox;
 	GtkWidget *ok_btn;
 	GtkWidget *cancel_btn;
-	GtkWidget *hsbox;
-	GtkWidget *statusbar;
 	gint top;
 	static GdkGeometry geometry;
 
 	window = gtkut_window_new(GTK_WINDOW_TOPLEVEL, "addressadd");
-	gtk_container_set_border_width( GTK_CONTAINER(window), 0 );
+	gtk_container_set_border_width( GTK_CONTAINER(window), VBOX_BORDER );
 	gtk_window_set_title( GTK_WINDOW(window), _("Add to address book") );
 	gtk_window_set_position( GTK_WINDOW(window), GTK_WIN_POS_MOUSE );
 	gtk_window_set_modal( GTK_WINDOW(window), TRUE );
@@ -183,16 +171,25 @@ static void addressadd_create( void ) {
 			  G_CALLBACK(addressadd_key_pressed), NULL );
 	g_signal_connect(G_OBJECT(window), "size_allocate",
 			 G_CALLBACK(addressadd_size_allocate_cb), NULL);
-
+	
+	hbox = gtk_hbox_new(FALSE, 10);
+	gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
 	vbox = gtk_vbox_new(FALSE, 8);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
-	gtk_container_set_border_width( GTK_CONTAINER(vbox), 8 );
+
+	picture = gtk_image_new();
+	gtk_box_pack_start(GTK_BOX(hbox), picture, FALSE, FALSE, 0);
 
 	table = gtk_table_new(3, 2, FALSE);
-	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
-	gtk_container_set_border_width( GTK_CONTAINER(table), 8 );
-	gtk_table_set_row_spacings(GTK_TABLE(table), 8);
-	gtk_table_set_col_spacings(GTK_TABLE(table), 8);
+	gtk_box_pack_start(GTK_BOX(hbox), table, TRUE, TRUE, 0);
+	gtk_table_set_row_spacings(GTK_TABLE(table), VSPACING_NARROW);
+	gtk_table_set_col_spacings(GTK_TABLE(table), HSPACING_NARROW);
+
+	frame = gtk_frame_new(_("Contact"));
+	gtk_frame_set_label_align(GTK_FRAME(frame), 0.01, 0.5);
+	gtk_container_add(GTK_CONTAINER(frame), hbox);
+	gtk_container_set_border_width( GTK_CONTAINER(frame), 4);
+	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
 
 	/* First row */
 	top = 0;
@@ -201,7 +198,7 @@ static void addressadd_create( void ) {
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
 	entry_name = gtk_entry_new();
-	/* gtk_entry_set_max_length(); */
+	gtk_widget_set_size_request(entry_name, 150, -1);
 	gtk_entry_set_text (GTK_ENTRY(entry_name),"");
 	
 	gtk_table_attach(GTK_TABLE(table), entry_name, 1, 2, top, (top + 1), GTK_FILL | GTK_EXPAND , 0, 0, 0);
@@ -213,7 +210,8 @@ static void addressadd_create( void ) {
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
 
 	label_addr = gtk_label_new("");
-	gtk_table_attach(GTK_TABLE(table), label_addr, 1, 2, top, (top + 1), GTK_FILL, 0, 0, 0);
+	gtk_widget_set_size_request(label_addr, 150, -1);
+	gtk_table_attach(GTK_TABLE(table), label_addr, 1, 2, top, (top + 1), GTK_FILL | GTK_EXPAND, 0, 0, 0);
 	gtk_misc_set_alignment(GTK_MISC(label_addr), 0, 0.5);
 
 	/* Third row */
@@ -223,13 +221,13 @@ static void addressadd_create( void ) {
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
 	entry_rems = gtk_entry_new();
-	/* gtk_entry_set_max_length(); */
+	gtk_widget_set_size_request(entry_rems, 150, -1);
 	gtk_table_attach(GTK_TABLE(table), entry_rems, 1, 2, top, (top + 1), GTK_FILL | GTK_EXPAND, 0, 0, 0);
-
+	
 	/* Address book/folder tree */
-	vlbox = gtk_vbox_new(FALSE, 8);
+	vlbox = gtk_vbox_new(FALSE, VBOX_BORDER);
 	gtk_box_pack_start(GTK_BOX(vbox), vlbox, TRUE, TRUE, 0);
-	gtk_container_set_border_width( GTK_CONTAINER(vlbox), 8 );
+	gtk_container_set_border_width( GTK_CONTAINER(vlbox), 4);
 
 	tree_win = gtk_scrolled_window_new( NULL, NULL );
 	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(tree_win),
@@ -247,18 +245,12 @@ static void addressadd_create( void ) {
 	gtk_ctree_set_indent( GTK_CTREE(tree_folder), CTREE_INDENT );
 	gtk_clist_set_auto_sort( GTK_CLIST(tree_folder), TRUE );
 
-	/* Status line */
-	hsbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_end(GTK_BOX(vbox), hsbox, FALSE, FALSE, BORDER_WIDTH);
-	statusbar = gtk_statusbar_new();
-	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, BORDER_WIDTH);
-
 	/* Button panel */
 	gtkut_stock_button_set_create(&hbbox, &cancel_btn, GTK_STOCK_CANCEL,
 				      &ok_btn, GTK_STOCK_OK,
 				      NULL, NULL);
 	gtk_box_pack_end(GTK_BOX(vbox), hbbox, FALSE, FALSE, 0);
-	gtk_container_set_border_width( GTK_CONTAINER(hbbox), 0 );
+	gtk_container_set_border_width( GTK_CONTAINER(hbbox), HSPACING_NARROW );
 	gtk_widget_grab_default(ok_btn);
 
 	g_signal_connect(G_OBJECT(ok_btn), "clicked",
@@ -272,7 +264,7 @@ static void addressadd_create( void ) {
 
 	if (!geometry.min_height) {
 		geometry.min_width = 300;
-		geometry.min_height = 400;
+		geometry.min_height = 350;
 	}
 
 	gtk_window_set_geometry_hints(GTK_WINDOW(window), NULL, &geometry,
@@ -280,17 +272,14 @@ static void addressadd_create( void ) {
 	gtk_widget_set_size_request(window, prefs_common.addressaddwin_width,
 				    prefs_common.addressaddwin_height);
 
-	gtk_widget_show_all(vbox);
-
 	addressadd_dlg.window        = window;
+	addressadd_dlg.picture       = picture;
 	addressadd_dlg.entry_name    = entry_name;
 	addressadd_dlg.label_address = label_addr;
 	addressadd_dlg.entry_remarks = entry_rems;
 	addressadd_dlg.tree_folder   = tree_folder;
 	addressadd_dlg.ok_btn        = ok_btn;
 	addressadd_dlg.cancel_btn    = cancel_btn;
-	addressadd_dlg.statusbar     = statusbar;
-	addressadd_dlg.status_cid    = gtk_statusbar_get_context_id( GTK_STATUSBAR(statusbar), "Address Add" );
 
 	gtk_widget_show_all( window );
 
@@ -377,7 +366,8 @@ static void addressadd_load_data( AddressIndex *addrIndex ) {
 	}
 }
 
-gboolean addressadd_selection( AddressIndex *addrIndex, const gchar *name, const gchar *address, const gchar *remarks ) {
+gboolean addressadd_selection( AddressIndex *addrIndex, const gchar *name, 
+		const gchar *address, const gchar *remarks, GdkPixbuf *picture ) {
 	gboolean retVal = FALSE;
 	ItemPerson *person = NULL;
 	FolderInfo *fi = NULL;
@@ -389,7 +379,6 @@ gboolean addressadd_selection( AddressIndex *addrIndex, const gchar *name, const
 	manage_window_set_transient(GTK_WINDOW(addressadd_dlg.window));
 
 	addressadd_dlg.fiSelected = NULL;
-	addressadd_status_show( "" );
 	addressadd_load_data( addrIndex );
 	gtk_clist_select_row( GTK_CLIST( addressadd_dlg.tree_folder ), 0, 0 );
 	gtk_widget_show(addressadd_dlg.window);
@@ -403,7 +392,12 @@ gboolean addressadd_selection( AddressIndex *addrIndex, const gchar *name, const
 		gtk_label_set_text( GTK_LABEL(addressadd_dlg.label_address ), address );
 	if( remarks )
 		gtk_entry_set_text( GTK_ENTRY(addressadd_dlg.entry_remarks ), remarks );
-
+	if( picture ) {
+		gtk_image_set_from_pixbuf(GTK_IMAGE(addressadd_dlg.picture), picture);
+		gtk_widget_show(GTK_WIDGET(addressadd_dlg.picture));
+	} else {
+		gtk_widget_hide(GTK_WIDGET(addressadd_dlg.picture));
+	}
 	gtk_main();
 	gtk_widget_hide( addressadd_dlg.window );
 
@@ -421,6 +415,20 @@ gboolean addressadd_selection( AddressIndex *addrIndex, const gchar *name, const
 							address, 
 							returned_remarks);
 			person->status = ADD_ENTRY;
+
+			if (picture) {
+				GError *error = NULL;
+				gchar *name = g_strconcat( get_rc_dir(), G_DIR_SEPARATOR_S, ADDRBOOK_DIR, G_DIR_SEPARATOR_S, 
+							ADDRITEM_ID(person), ".png", NULL );
+				gdk_pixbuf_save(picture, name, "png", &error, NULL);
+				if (error) {
+					g_warning(_("Failed to save image: \n%s"),
+							error->message);
+					g_error_free(error);
+				}
+				addritem_person_set_picture( person, ADDRITEM_ID(person) ) ;
+				g_free( name );
+			}
 #ifdef USE_LDAP
 			if (fi->book->type == ADBOOKTYPE_LDAP) {
 				LdapServer *server = (LdapServer *) fi->book;
