@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /*
@@ -217,6 +217,7 @@ ItemPerson *addritem_create_item_person( void ) {
 	ADDRITEM_NAME(person) = NULL;
 	ADDRITEM_PARENT(person) = NULL;
 	ADDRITEM_SUBTYPE(person) = 0;
+	person->picture = NULL;
 	person->firstName = NULL;
 	person->lastName = NULL;
 	person->nickName = NULL;
@@ -240,6 +241,7 @@ ItemPerson *addritem_copy_item_person( ItemPerson *item ) {
 	if( item ) {
 		itemNew = addritem_create_item_person();
 		ADDRITEM_NAME(itemNew) = g_strdup( ADDRITEM_NAME(item) );
+		itemNew->picture = g_strdup( ADDRITEM_ID(itemNew) );
 		itemNew->firstName = g_strdup( item->firstName );
 		itemNew->lastName = g_strdup( item->lastName );
 		itemNew->nickName = g_strdup( item->nickName );
@@ -249,6 +251,34 @@ ItemPerson *addritem_copy_item_person( ItemPerson *item ) {
 	return itemNew;
 }
 
+/**
+ * Specify picture for person object.
+ * \param person Person object.
+ * \param value Picture.
+ */
+void addritem_person_set_picture( ItemPerson *person, const gchar *value ) {
+	if (!value || g_utf8_validate(value, -1, NULL))
+		person->picture = mgu_replace_string( person->picture, value );
+	else {
+		gchar *out = conv_codeset_strdup(value, 
+				conv_get_locale_charset_str_no_utf8(),
+				CS_INTERNAL);
+		if (out)
+			person->picture = mgu_replace_string( person->picture, out );
+		g_free(out);
+	}
+}
+
+/**
+ * Get picture for person object.
+ * \param person Person object.
+ * \param value Picture.
+ */
+gchar *addritem_person_get_picture( ItemPerson *person) {
+	if (person->picture)
+		return g_strdup(person->picture);
+	return NULL;
+}
 /**
  * Specify first name for person object.
  * \param person Person object.
@@ -385,6 +415,7 @@ void addritem_free_item_person( ItemPerson *person ) {
 	/* Free internal stuff */
 	g_free( ADDRITEM_ID(person) );
 	g_free( ADDRITEM_NAME(person) );
+	g_free( person->picture );
 	g_free( person->firstName );
 	g_free( person->lastName );
 	g_free( person->nickName );
@@ -397,6 +428,7 @@ void addritem_free_item_person( ItemPerson *person ) {
 	ADDRITEM_NAME(person) = NULL;
 	ADDRITEM_PARENT(person) = NULL;
 	ADDRITEM_SUBTYPE(person) = 0;
+	person->picture = NULL;
 	person->firstName = NULL;
 	person->lastName = NULL;
 	person->nickName = NULL;
@@ -447,6 +479,7 @@ void addritem_print_item_person( ItemPerson *person, FILE *stream ) {
 	fprintf( stream, "\tt/uid: %d : '%s'\n", ADDRITEM_TYPE(person), ADDRITEM_ID(person) );
 	fprintf( stream, "\tsubty: %d\n", ADDRITEM_SUBTYPE(person) );
 	fprintf( stream, "\tcommn: '%s'\n", ADDRITEM_NAME(person) );
+	fprintf( stream, "\tphoto: '%s'\n", person->picture );
 	fprintf( stream, "\tfirst: '%s'\n", person->firstName );
 	fprintf( stream, "\tlast : '%s'\n", person->lastName );
 	fprintf( stream, "\tnick : '%s'\n", person->nickName );
@@ -550,6 +583,27 @@ void addritem_person_add_attribute(
 	person->listAttrib = g_list_append( person->listAttrib, attrib );
 }
 
+/**
+ * Remove user attribute from specified person.
+ * \param  person Person.
+ * \param  attrib Attribute to remove.
+ */
+void addritem_person_remove_attribute(	ItemPerson *person, const gchar *attrib ) {
+	g_return_if_fail( person != NULL || attrib != NULL );
+	GList *attrib_list;
+
+	attrib_list = person->listAttrib;
+	while (attrib_list) {
+		UserAttribute *user_attrib = attrib_list->data;
+		if (strcmp(user_attrib->name, attrib) == 0) {
+			person->listAttrib = g_list_remove(person->listAttrib, (gconstpointer) user_attrib);
+			break;
+		}
+		attrib_list = g_list_next(attrib_list);
+	}
+}
+
+
 /**
  * Create new address book group object.
  * \return Initialized group object.
@@ -962,10 +1016,10 @@ GList *addritem_folder_get_person_list( ItemFolder *folder ) {
 	node = folder->listPerson;
 	while( node ) {
 		ItemPerson *person = node->data;
-		list = g_list_append( list, person );
+		list = g_list_prepend( list, person );
 		node = g_list_next( node );
 	}
-	return list;
+	return g_list_reverse(list);
 }
 
 /**
@@ -985,10 +1039,10 @@ GList *addritem_folder_get_group_list( ItemFolder *folder ) {
 	node = folder->listGroup;
 	while( node ) {
 		ItemGroup *group = node->data;
-		list = g_list_append( list, group );
+		list = g_list_prepend( list, group );
 		node = g_list_next( node );
 	}
-	return list;
+	return g_list_reverse(list);
 }
 
 /**
