@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifndef __UTILS_H__
@@ -108,6 +108,14 @@ gint g_chmod	(const gchar	*path,
 	if (change_dir(dir) < 0) return val; \
 }
 
+#define CHDIR_EXEC_CODE_RETURN_VAL_IF_FAIL(dir, val, code) \
+{ \
+	if (change_dir(dir) < 0) { \
+		code \
+		return val; \
+	} \
+}
+
 #define Xalloca(ptr, size, iffail) \
 { \
 	if ((ptr = alloca(size)) == NULL) { \
@@ -172,7 +180,7 @@ gint g_chmod	(const gchar	*path,
 
 #define FILE_OP_ERROR(file, func) \
 { \
-	fprintf(stderr, "%s: ", file); \
+	g_printerr("%s: ", file); \
 	fflush(stderr); \
 	perror(func); \
 }
@@ -204,11 +212,18 @@ gboolean debug_get_mode		(void);
 #define Str(x)	#x
 #define Xstr(x)	Str(x)
 
+
+/* System related stuff.  */
+
+gboolean superuser_p (void);
+
+
+/* String utilities.  */
+
 void list_free_strings		(GList		*list);
 void slist_free_strings		(GSList		*list);
 
 void hash_free_strings		(GHashTable	*table);
-void hash_free_value_mem	(GHashTable	*table);
 
 gint str_case_equal		(gconstpointer	 v,
 				 gconstpointer	 v2);
@@ -223,11 +238,6 @@ gboolean str_find		(const gchar	*haystack,
 				 const gchar	*needle);
 gboolean str_case_find		(const gchar	*haystack,
 				 const gchar	*needle);
-gboolean str_find_equal		(const gchar	*haystack,
-				 const gchar	*needle);
-gboolean str_case_find_equal	(const gchar	*haystack,
-				 const gchar	*needle);
-
 /* number-string conversion */
 gint to_number			(const gchar *nstr);
 gchar *itos_buf			(gchar	     *nstr,
@@ -246,7 +256,7 @@ gchar *strretchomp	(gchar		*str);
 gchar *strtailchomp	(gchar		*str,
 			 gchar		 tail_char);
 gchar *strcrchomp	(gchar		*str);
-void file_strip_crs	(const gchar 	*file);
+gint file_strip_crs	(const gchar 	*file);
 gchar *strcasestr	(const gchar	*haystack,
 			 const gchar	*needle);
 gpointer my_memmem	(gconstpointer	 haystack,
@@ -265,8 +275,6 @@ gint subject_compare			(const gchar	*s1,
 					 const gchar	*s2);
 gint subject_compare_for_sort		(const gchar	*s1,
 					 const gchar	*s2);
-void trim_subject_for_compare		(gchar		*str);
-void trim_subject_for_sort		(gchar		*str);
 void trim_subject			(gchar		*str);
 void eliminate_parenthesis		(gchar		*str,
 					 gchar		 op,
@@ -275,27 +283,15 @@ void extract_parenthesis		(gchar		*str,
 					 gchar		 op,
 					 gchar		 cl);
 
-void extract_parenthesis_with_skip_quote	(gchar		*str,
-						 gchar		 quote_chr,
-						 gchar		 op,
-						 gchar		 cl);
-
-void eliminate_quote			(gchar		*str,
-					 gchar		 quote_chr);
 void extract_quote			(gchar		*str,
 					 gchar		 quote_chr);
 void eliminate_address_comment		(gchar		*str);
 gchar *strchr_with_skip_quote		(const gchar	*str,
 					 gint		 quote_chr,
 					 gint		 c);
-gchar *strrchr_with_skip_quote		(const gchar	*str,
-					 gint		 quote_chr,
-					 gint		 c);
 void extract_address			(gchar		*str);
 void extract_list_id_str		(gchar		*str);
 
-GSList *slist_concat_unique		(GSList		*first,
-					 GSList		*second);
 GSList *address_list_append		(GSList		*addr_list,
 					 const gchar	*str);
 GSList *address_list_append_with_comments(GSList	*addr_list,
@@ -316,12 +312,11 @@ void unfold_line			(gchar		*str);
 void subst_char				(gchar		*str,
 					 gchar		 orig,
 					 gchar		 subst);
-void subst_chars			(gchar		*str,
-					 gchar		*orig,
-					 gchar		 subst);
+void subst_chars			(gchar 		*str, 	
+					 gchar 		*orig, 
+					 gchar 		subst);
 void subst_for_filename			(gchar		*str);
 void subst_for_shellsafe_filename	(gchar		*str);
-gboolean is_header_line			(const gchar	*str);
 gboolean is_ascii_str			(const gchar	*str);
 gint get_quote_level			(const gchar	*str,
 					 const gchar	*quote_chars);
@@ -329,16 +324,6 @@ gint check_line_length			(const gchar	*str,
 					 gint		 max_chars,
 					 gint		*line);
 
-gchar *strstr_with_skip_quote		(const gchar	*haystack,
-					 const gchar	*needle);
-gchar *strchr_parenthesis_close		(const gchar	*str,
-					 gchar		 op,
-					 gchar		 cl);
-
-gchar **strsplit_parenthesis		(const gchar	*str,
-					 gchar		 op,
-					 gchar		 cl,
-					 gint		 max_tokens);
 gchar **strsplit_with_quote		(const gchar	*str,
 					 const gchar	*delim,
 					 gint		 max_tokens);
@@ -358,23 +343,24 @@ void decode_uri_with_plus		(gchar 		*decoded_uri,
 					 const gchar 	*encoded_uri, 
 					 gboolean 	 with_plus);
 gint scan_mailto_url			(const gchar	*mailto,
+					 gchar	       **from,
 					 gchar	       **to,
 					 gchar	       **cc,
 					 gchar	       **bcc,
 					 gchar	       **subject,
 					 gchar	       **body,
-					 gchar	       **attach);
+					 gchar	       ***attach);
 
 /* return static strings */
 const gchar *get_home_dir		(void);
 const gchar *get_rc_dir			(void);
+void  set_rc_dir			(const gchar *dir);
+gboolean rc_dir_is_alt			(void);
 const gchar *get_mail_base_dir		(void);
 const gchar *get_news_cache_dir		(void);
 const gchar *get_imap_cache_dir		(void);
-const gchar *get_mbox_cache_dir		(void);
 const gchar *get_mime_tmp_dir		(void);
 const gchar *get_template_dir		(void);
-const gchar *get_header_cache_dir	(void);
 const gchar *get_plugin_dir             (void);
 const gchar *get_tmp_dir		(void);
 const gchar *get_locale_dir		(void);
@@ -384,7 +370,6 @@ const gchar *get_domain_name		(void);
 /* file / directory handling */
 off_t get_file_size		(const gchar	*file);
 off_t get_file_size_as_crlf	(const gchar	*file);
-off_t get_left_file_size	(FILE		*fp);
 
 time_t get_file_mtime		(const gchar *file);
 
@@ -394,7 +379,6 @@ gboolean is_relative_filename   (const gchar *file);
 gboolean is_dir_exist		(const gchar	*dir);
 gboolean is_file_entry_exist	(const gchar	*file);
 gboolean dirent_is_regular_file	(struct dirent	*d);
-gboolean dirent_is_directory	(struct dirent	*d);
 
 #define is_file_exist(file)		file_exist(file, FALSE)
 #define is_file_or_fifo_exist(file)	file_exist(file, TRUE)
@@ -409,8 +393,6 @@ gint remove_numbered_files	(const gchar	*dir,
 gint remove_numbered_files_not_in_list(const gchar *dir,
 				       GSList *numberlist);
 gint remove_all_numbered_files	(const gchar	*dir);
-gint remove_expired_files	(const gchar	*dir,
-				 guint		 hours);
 gint remove_dir_recursive	(const gchar	*dir);
 gint append_file		(const gchar	*src,
 				 const gchar	*dest,
@@ -438,9 +420,6 @@ gchar *canonicalize_str		(const gchar	*str);
 gint canonicalize_file		(const gchar	*src,
 				 const gchar	*dest);
 gint canonicalize_file_replace	(const gchar	*file);
-gint uncanonicalize_file	(const gchar	*src,
-				 const gchar	*dest);
-gint uncanonicalize_file_replace(const gchar	*file);
 
 gchar *normalize_newlines	(const gchar	*str);
 
@@ -456,6 +435,8 @@ gint str_write_to_file		(const gchar	*str,
 				 const gchar	*file);
 gchar *file_read_to_str		(const gchar	*file);
 gchar *file_read_stream_to_str	(FILE		*fp);
+gchar *file_read_to_str_no_recode(const gchar *file);
+gchar *file_read_stream_to_str_no_recode(FILE *fp);
 
 char *fgets_crlf(char *buf, int size, FILE *stream);
 
@@ -476,10 +457,6 @@ gchar *tzoffset			(time_t		*now);
 void get_rfc822_date		(gchar		*buf,
 				 gint		 len);
 
-size_t my_strftime		(gchar			*s,
-				 size_t			 max,
-				 const gchar		*format,
-				 const struct tm	*tm);
 size_t fast_strftime		(gchar 			*buf, 
 				 gint 			 buflen, 
 				 const gchar 		*format, 
@@ -494,16 +471,13 @@ void * subject_table_lookup(GHashTable *subject_table, gchar * subject);
 void subject_table_insert(GHashTable *subject_table, gchar * subject,
 			  void * data);
 void subject_table_remove(GHashTable *subject_table, gchar * subject);
+void utils_free_regex(void);
 gint subject_get_prefix_length (const gchar *subject);
 
 /* quoting recognition */
 const gchar * line_has_quote_char	(const gchar *str,
 					 const gchar *quote_chars);
-const gchar * line_has_quote_char_last	(const gchar *str,
-					 const gchar *quote_chars);
 
-guint g_stricase_hash	(gconstpointer gptr);
-gint g_stricase_equal	(gconstpointer gptr1, gconstpointer gptr2);
 gint g_int_compare	(gconstpointer a, gconstpointer b);
 
 gchar *generate_msgid		(gchar *buf, gint len);
