@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -317,7 +317,7 @@ static void crash_save_crash_log(GtkButton *button, const gchar *text)
  */
 static void crash_create_bug_report(GtkButton *button, const gchar *data)
 {
-	open_uri(BUGZILLA_URI, prefs_common.uri_cmd);
+	open_uri(BUGZILLA_URI, prefs_common_get_uri_cmd());
 }
 
 /*!
@@ -361,7 +361,7 @@ static void crash_debug(unsigned long crash_pid,
 		dup(choutput[1]);
 		close(choutput[0]);
 		if (-1 == execvp("gdb", argp)) 
-			puts("error execvp\n");
+			g_print("error execvp\n");
 	} else {
 		char buf[100];
 		int r;
@@ -372,7 +372,7 @@ static void crash_debug(unsigned long crash_pid,
 		 * make it non blocking
 		 */
 		if (-1 == fcntl(choutput[0], F_SETFL, O_NONBLOCK))
-			puts("set to non blocking failed\n");
+			g_print("set to non blocking failed\n");
 
 		/*
 		 * get the output
@@ -418,6 +418,9 @@ static const gchar *get_compiled_in_features(void)
 #if USE_OPENSSL
 		   " OpenSSL"
 #endif
+#if USE_GNUTLS
+		   " GnuTLS"
+#endif
 #if USE_LDAP
 		   " LDAP"
 #endif
