@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -31,6 +31,7 @@
 #include <libetpan/mailsem.h>
 #include <semaphore.h>
 #include <unistd.h>
+#include <fcntl.h>
 
 #include "etpan-errors.h"
 
@@ -196,6 +197,7 @@ struct etpan_thread_op * etpan_thread_op_new(void)
   op->result = NULL;
   op->finished = 0;
   op->imap = NULL;
+  op->nntp = NULL;
 
   r = pthread_mutex_init(&op->lock, NULL);
   if (r != 0)
