@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "defs.h"
@@ -198,7 +198,7 @@ FolderItem *foldersel_folder_sel(Folder *cur_folder, FolderSelectionType type,
 	gtk_tree_store_clear(tree_store);
 
 	if (!cancelled &&
-	    selected_item && selected_item->path && !selected_item->no_select) {
+	    selected_item && selected_item->path) {
 		folder_item = selected_item;
 		return folder_item;
 	} else
@@ -494,7 +494,7 @@ static gboolean foldersel_selected(GtkTreeSelection *selection,
 			   FOLDERSEL_FOLDERITEM, &item, -1);
 
 	selected_item = item;
-	if (selected_item && selected_item->path && !selected_item->no_select) {
+	if (selected_item && selected_item->path) {
 		gchar *id;
 		id = folder_item_get_identifier(selected_item);
 		gtk_entry_set_text(GTK_ENTRY(entry), id);
