@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,8 +14,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -157,7 +157,13 @@ static gchar *get_part_as_string(MimeInfo *mimeinfo)
 			textdata = tmp;
 		}
 	}
+
 	if (textdata && mimeinfo->offset && 
+	    mimeinfo->offset+ mimeinfo->length <= g_utf8_strlen(textdata, -1)) {
+		real_data = g_strdup(textdata + mimeinfo->offset);
+		g_free(textdata);
+		textdata = real_data;
+	} else if (textdata && mimeinfo->offset && 
 	    mimeinfo->offset+ mimeinfo->length <= strlen(textdata)) {
 		real_data = g_strdup(textdata + mimeinfo->offset);
 		real_data[mimeinfo->length] = '\0';
@@ -234,16 +240,20 @@ static gint pgpinline_check_signature(MimeInfo *mimeinfo)
 
 	g_return_val_if_fail(mimeinfo != NULL, 0);
 
-	if (procmime_mimeinfo_parent(mimeinfo) == NULL)
+	if (procmime_mimeinfo_parent(mimeinfo) == NULL) {
+		privacy_set_error(_("Incorrect part"));
 		return 0; /* not parent */
-	if (mimeinfo->type != MIMETYPE_TEXT)
+	}
+	if (mimeinfo->type != MIMETYPE_TEXT) {
+		privacy_set_error(_("Not a text part"));
+		debug_print("type %d\n", mimeinfo->type);
 		return 0;
-
+	}
 	g_return_val_if_fail(mimeinfo->privacy != NULL, 0);
 	data = (PrivacyDataPGP *) mimeinfo->privacy;
 
 	textdata = get_part_as_string(mimeinfo);
-	
+
 	if (!textdata) {
 		g_free(textdata);
 		privacy_set_error(_("Couldn't get text data."));
@@ -436,38 +446,87 @@ static MimeInfo *pgpinline_decrypt(MimeInfo *mimeinfo)
 	if (src_codeset == NULL)
 		src_codeset = CS_ISO_8859_1;
 		
-	fprintf(dstfp, "MIME-Version: 1.0\r\n"
+	if (fprintf(dstfp, "MIME-Version: 1.0\r\n"
 			"Content-Type: text/plain; charset=%s\r\n"
 			"Content-Transfer-Encoding: 8bit\r\n"
 			"\r\n",
-			src_codeset);
+			src_codeset) < 0) {
+        	FILE_OP_ERROR(fname, "fprintf");
+		privacy_set_error(_("Couldn't write to decrypted file %s"), fname);
+        	g_free(fname);
+        	gpgme_data_release(plain);
+		gpgme_release(ctx);
+		return NULL;
+	}
 
 	/* Store any part before encrypted text */
 	pos = strstr(textdata, begin_indicator);
 	if (pos != NULL && (pos - textdata) > 0) {
-	    fwrite(textdata, pos - textdata, 1, dstfp);
+	    if (fwrite(textdata, 1, pos - textdata, dstfp) < pos - textdata) {
+        	FILE_OP_ERROR(fname, "fwrite");
+		privacy_set_error(_("Couldn't write to decrypted file %s"), fname);
+        	g_free(fname);
+        	gpgme_data_release(plain);
+		gpgme_release(ctx);
+		return NULL;
+	    }
 	}
 	
-	fwrite(_("\n--- Start of PGP/Inline encrypted data ---\n"), 
+	if (fwrite(_("\n--- Start of PGP/Inline encrypted data ---\n"), 1,
 		strlen(_("\n--- Start of PGP/Inline encrypted data ---\n")), 
-		1, dstfp);
+		dstfp) < strlen(_("\n--- Start of PGP/Inline encrypted data ---\n"))) {
+        	FILE_OP_ERROR(fname, "fwrite");
+		privacy_set_error(_("Couldn't write to decrypted file %s"), fname);
+        	g_free(fname);
+        	gpgme_data_release(plain);
+		gpgme_release(ctx);
+		return NULL;
+	}
 	chars = sgpgme_data_release_and_get_mem(plain, &len);
-	if (len > 0)
-		fwrite(chars, len, 1, dstfp);
-
+	if (len > 0) {
+		if (fwrite(chars, 1, len, dstfp) < len) {
+        		FILE_OP_ERROR(fname, "fwrite");
+			privacy_set_error(_("Couldn't write to decrypted file %s"), fname);
+        		g_free(fname);
+        		gpgme_data_release(plain);
+			gpgme_release(ctx);
+			return NULL;
+		}
+	}
 	/* Store any part after encrypted text */
-	fwrite(_("--- End of PGP/Inline encrypted data ---\n"), 
+	if (fwrite(_("--- End of PGP/Inline encrypted data ---\n"), 1,
 		strlen(_("--- End of PGP/Inline encrypted data ---\n")), 
-		1, dstfp);
+		dstfp) < strlen(_("--- End of PGP/Inline encrypted data ---\n"))) {
+        		FILE_OP_ERROR(fname, "fwrite");
+			privacy_set_error(_("Couldn't write to decrypted file %s"), fname);
+        		g_free(fname);
+        		gpgme_data_release(plain);
+			gpgme_release(ctx);
+			return NULL;
+	}
 	if (pos != NULL) {
 	    pos = strstr(pos, end_indicator);
 	    if (pos != NULL && *pos != '\0') {
 		pos += strlen(end_indicator);
-		fwrite(pos, strlen(pos), 1, dstfp);
+		if (fwrite(pos, 1, strlen(pos), dstfp) < strlen(pos)) {
+        		FILE_OP_ERROR(fname, "fwrite");
+			privacy_set_error(_("Couldn't write to decrypted file %s"), fname);
+        		g_free(fname);
+        		gpgme_data_release(plain);
+			gpgme_release(ctx);
+			return NULL;
+		}
 	    }
 	}
 
-	fclose(dstfp);
+	if (fclose(dstfp) == EOF) {
+        	FILE_OP_ERROR(fname, "fclose");
+		privacy_set_error(_("Couldn't close decrypted file %s"), fname);
+        	g_free(fname);
+        	gpgme_data_release(plain);
+		gpgme_release(ctx);
+		return NULL;
+	}
 	
 	parseinfo = procmime_scan_file(fname);
 	g_free(fname);
