@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -45,7 +45,7 @@ static gint pop3_greeting_recv		(Pop3Session *session,
 static gint pop3_getauth_user_send	(Pop3Session *session);
 static gint pop3_getauth_pass_send	(Pop3Session *session);
 static gint pop3_getauth_apop_send	(Pop3Session *session);
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 static gint pop3_stls_send		(Pop3Session *session);
 static gint pop3_stls_recv		(Pop3Session *session);
 #endif
@@ -90,6 +90,7 @@ static gint pop3_session_recv_msg		(Session	*session,
 static gint pop3_session_recv_data_finished	(Session	*session,
 						 guchar		*data,
 						 guint		 len);
+static void pop3_get_uidl_table(PrefsAccount *ac_prefs, Pop3Session *session);
 
 static gint pop3_greeting_recv(Pop3Session *session, const gchar *msg)
 {
@@ -99,7 +100,7 @@ static gint pop3_greeting_recv(Pop3Session *session, const gchar *msg)
 	return PS_SUCCESS;
 }
 
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 static gint pop3_stls_send(Pop3Session *session)
 {
 	session->state = POP3_STLS;
@@ -161,7 +162,7 @@ static gint pop3_getauth_apop_send(Pop3Session *session)
 	*(end + 1) = '\0';
 
 	if (!is_ascii_str(start)) {
-		log_error(LOG_PROTOCOL, _("Timestamp syntax error in greeting (not ascii)\n"));
+		log_error(LOG_PROTOCOL, _("Timestamp syntax error in greeting (not ASCII)\n"));
 		session->error_val = PS_PROTOCOL;
 		return -1;
 	}
@@ -264,8 +265,8 @@ static gint pop3_getrange_uidl_recv(Pop3Session *session, const gchar *data,
 
 		session->msg[num].uidl = g_strdup(id);
 
-		recv_time = (time_t)g_hash_table_lookup(
-			   		session->uidl_table, id);
+		recv_time = (time_t)(GPOINTER_TO_INT(g_hash_table_lookup(
+			   		session->uidl_table, id)));
 		session->msg[num].recv_time = recv_time;
 
 		if (recv_time != RECV_TIME_NONE) {
@@ -274,8 +275,8 @@ static gint pop3_getrange_uidl_recv(Pop3Session *session, const gchar *data,
 			debug_print("num %d uidl %s: unknown\n", num, id);
 		}
 
-		partial_recv = (gint)g_hash_table_lookup(
-					session->partial_recv_table, id);
+		partial_recv = (gint)(GPOINTER_TO_INT(g_hash_table_lookup(
+					session->partial_recv_table, id)));
 
 		if (recv_time != RECV_TIME_NONE
 		|| partial_recv != POP3_TOTALLY_RECEIVED) {
@@ -567,7 +568,7 @@ static void pop3_session_destroy(Session *session)
 	g_free(pop3_session->error_msg);
 }
 
-void pop3_get_uidl_table(PrefsAccount *ac_prefs, Pop3Session *session)
+static void pop3_get_uidl_table(PrefsAccount *ac_prefs, Pop3Session *session)
 {
 	GHashTable *table;
 	GHashTable *partial_recv_table;
@@ -734,8 +735,12 @@ static gint pop3_write_msg_to_file(const gchar *file, const gchar *data,
 		FILE_OP_ERROR(file, "chmod");
 
 	if (prefix != NULL) {
-		fprintf(fp, "%s", prefix);
-		fprintf(fp, "\n");
+		if (fprintf(fp, "%s\n", prefix) < 0) {
+			FILE_OP_ERROR(file, "fprintf");
+			fclose(fp);
+			g_unlink(file);
+			return -1;
+		}
 	}
 	
 	/* +------------------+----------------+--------------------------+ *
@@ -874,7 +879,7 @@ static Pop3ErrorValue pop3_ok(Pop3Session *session, const gchar *msg)
 			ok = PS_ERROR;
 		} else {
 			switch (session->state) {
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 			case POP3_STLS:
 				log_error(LOG_PROTOCOL, _("couldn't start TLS session\n"));
 				ok = PS_ERROR;
@@ -901,7 +906,7 @@ static Pop3ErrorValue pop3_ok(Pop3Session *session, const gchar *msg)
 
 		g_free(session->error_msg);
 		session->error_msg = g_strdup(msg);
-		fprintf(stderr, "POP3: %s\n", msg);
+		g_printerr("POP3: %s\n", msg);
 	} else
 		ok = PS_PROTOCOL;
 
@@ -938,7 +943,7 @@ static gint pop3_session_recv_msg(Session *session, const gchar *msg)
 	case POP3_READY:
 	case POP3_GREETING:
 		pop3_greeting_recv(pop3_session, body);
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 		if (pop3_session->ac_prefs->ssl_pop == SSL_STARTTLS)
 			val = pop3_stls_send(pop3_session);
 		else
@@ -948,7 +953,7 @@ static gint pop3_session_recv_msg(Session *session, const gchar *msg)
 		else
 			val = pop3_getauth_user_send(pop3_session);
 		break;
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 	case POP3_STLS:
 		if (pop3_stls_recv(pop3_session) != PS_SUCCESS)
 			return -1;
