@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,6 +42,8 @@
 #include "gtkutils.h"
 #include "utils.h"
 
+static void prefs_folder_column_set_config(FolderColumnState *state);
+
 enum {
 	SUMCOL_NAME,
 	SUMCOL_TYPE,
@@ -143,6 +145,13 @@ static void drag_data_received	(GtkTreeView *tree_view,
 				 guint time, 
 				 GtkTreeModel *model);
 
+static void prefs_folder_column_shown_set_btn_sensitivity(void);
+static void prefs_folder_column_shown_set_active(const gboolean active);
+static void prefs_folder_column_stock_set_active(const gboolean active);
+static void prefs_folder_column_shown_sel_changed(void);
+static void prefs_folder_column_stock_sel_changed(void);
+
+
 void prefs_folder_column_open(void)
 {
 	inc_lock();
@@ -249,6 +258,9 @@ static void prefs_folder_column_create(void)
 				       
 	stock_list_view = prefs_folder_column_list_view_create
 				(_("Hidden columns"));
+	g_signal_connect(G_OBJECT(stock_list_view), "cursor-changed",
+			   G_CALLBACK(prefs_folder_column_stock_sel_changed),
+			   NULL);
 	gtk_widget_show(stock_list_view);
 	gtk_container_add(GTK_CONTAINER(scrolledwin), stock_list_view);
 
@@ -278,6 +290,9 @@ static void prefs_folder_column_create(void)
 
 	shown_list_view = prefs_folder_column_list_view_create
 				(_("Displayed columns"));
+	g_signal_connect(G_OBJECT(shown_list_view), "cursor-changed",
+			   G_CALLBACK(prefs_folder_column_shown_sel_changed),
+			   NULL);
 	gtk_widget_show(shown_list_view);
 	gtk_container_add(GTK_CONTAINER(scrolledwin), shown_list_view);
 
@@ -333,7 +348,6 @@ static void prefs_folder_column_create(void)
 	g_signal_connect(G_OBJECT(cancel_btn), "clicked",
 			 G_CALLBACK(prefs_folder_column_cancel), NULL);
 
-
 	folder_col.window      = window;
 	folder_col.add_btn     = add_btn;
 	folder_col.remove_btn  = remove_btn;
@@ -343,6 +357,9 @@ static void prefs_folder_column_create(void)
 	folder_col.cancel_btn  = cancel_btn;
 	folder_col.stock_list_view = stock_list_view;
 	folder_col.shown_list_view = shown_list_view;
+	
+	prefs_folder_column_shown_set_active(FALSE);
+	prefs_folder_column_stock_set_active(FALSE);
 }
 
 FolderColumnState *prefs_folder_column_get_config(void)
@@ -370,7 +387,7 @@ FolderColumnState *prefs_folder_column_get_config(void)
 	return state;
 }
 
-void prefs_folder_column_set_config(FolderColumnState *state)
+static void prefs_folder_column_set_config(FolderColumnState *state)
 {
 	FolderColumnType type;
 	gint pos;
@@ -496,6 +513,8 @@ static void prefs_folder_column_add(void)
 	gtk_tree_selection_select_iter(gtk_tree_view_get_selection
 		(GTK_TREE_VIEW(folder_col.shown_list_view)),
 		 &shown_add);
+	prefs_folder_column_shown_set_active(TRUE);
+	prefs_folder_column_stock_set_active(FALSE);
 }
 
 static void prefs_folder_column_remove(void)
@@ -543,6 +562,8 @@ static void prefs_folder_column_remove(void)
 	gtk_tree_selection_select_iter(gtk_tree_view_get_selection
 		(GTK_TREE_VIEW(folder_col.stock_list_view)),
 		&stock_add);
+	prefs_folder_column_shown_set_active(FALSE);
+	prefs_folder_column_stock_set_active(TRUE);
 }
 
 static void prefs_folder_column_up(void)
@@ -580,6 +601,7 @@ static void prefs_folder_column_up(void)
 	gtk_tree_path_free(prev);
 
 	gtk_list_store_swap(shown_store, &iprev, &isel);
+	prefs_folder_column_shown_set_btn_sensitivity();
 }
 
 static void prefs_folder_column_down(void)
@@ -602,11 +624,14 @@ static void prefs_folder_column_down(void)
 		return;
 
 	gtk_list_store_swap(shown_store, &next, &sel);
+	prefs_folder_column_shown_set_btn_sensitivity();	
 }
 
 static void prefs_folder_column_set_to_default(void)
 {
 	prefs_folder_column_set_dialog(default_state);
+	prefs_folder_column_shown_set_active(FALSE);
+	prefs_folder_column_stock_set_active(FALSE);
 }
 
 static void prefs_folder_column_ok(void)
@@ -872,6 +897,83 @@ static void drag_data_received(GtkTreeView *tree_view, GdkDragContext *context,
 		gtk_drag_finish(context, TRUE, FALSE, time);
 	}
 
+	prefs_folder_column_shown_set_active(FALSE);
+	prefs_folder_column_stock_set_active(FALSE);
+	
 	/* XXXX: should we call gtk_drag_finish() for other code paths? */
 }
 
+static void prefs_folder_column_shown_set_btn_sensitivity(void)
+{
+	GtkTreeModel *model = GTK_TREE_MODEL(gtk_tree_view_get_model(
+		GTK_TREE_VIEW(folder_col.shown_list_view)));
+	GtkTreeSelection *selection = gtk_tree_view_get_selection(
+		GTK_TREE_VIEW(folder_col.shown_list_view));
+	GtkTreeIter iter;
+	GtkTreePath *path;
+	
+	if(!gtk_tree_selection_get_selected(selection, NULL, &iter)) {
+		gtk_widget_set_sensitive(folder_col.remove_btn, FALSE);
+		gtk_widget_set_sensitive(folder_col.up_btn, FALSE);
+		gtk_widget_set_sensitive(folder_col.down_btn, FALSE);
+		return;
+	}
+	
+	path = gtk_tree_model_get_path(model, &iter);
+
+	gtk_widget_set_sensitive(folder_col.up_btn, gtk_tree_path_prev(path));
+	gtk_widget_set_sensitive(folder_col.down_btn,
+				 gtk_tree_model_iter_next(model, &iter));
+	gtk_tree_path_free(path);
+}
+
+static void prefs_folder_column_shown_set_active(const gboolean active)
+{
+	GtkTreeSelection *selection = NULL;
+	
+	gtk_widget_set_sensitive(folder_col.remove_btn, active);
+	
+	if(active == FALSE) {
+		selection = gtk_tree_view_get_selection(
+			GTK_TREE_VIEW(folder_col.shown_list_view));
+		gtk_tree_selection_unselect_all(selection);
+		
+		gtk_widget_set_sensitive(folder_col.up_btn, FALSE);
+		gtk_widget_set_sensitive(folder_col.down_btn, FALSE);
+	} else {
+		prefs_folder_column_shown_set_btn_sensitivity();
+	}
+}
+
+static void prefs_folder_column_stock_set_active(const gboolean active)
+{
+	GtkTreeSelection *selection = NULL;
+	
+	gtk_widget_set_sensitive(folder_col.add_btn, active);
+	
+	if(active == FALSE) {
+		selection = gtk_tree_view_get_selection(
+			GTK_TREE_VIEW(folder_col.stock_list_view));
+		gtk_tree_selection_unselect_all(selection);
+	}
+}
+
+static void prefs_folder_column_stock_sel_changed(void)
+{
+	GtkTreeSelection *selection = gtk_tree_view_get_selection(
+		GTK_TREE_VIEW(folder_col.stock_list_view));
+	prefs_folder_column_stock_set_active(
+		(selection != NULL) ? TRUE : FALSE);
+	prefs_folder_column_shown_set_active(
+		(selection != NULL) ? FALSE : TRUE);
+}
+
+static void prefs_folder_column_shown_sel_changed(void)
+{
+	GtkTreeSelection *selection = gtk_tree_view_get_selection(
+		GTK_TREE_VIEW(folder_col.shown_list_view));
+	prefs_folder_column_shown_set_active(
+		(selection != NULL) ? TRUE : FALSE);
+	prefs_folder_column_stock_set_active(
+		(selection != NULL) ? FALSE : TRUE);
+}
