@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "defs.h"
@@ -45,6 +45,7 @@
 #include "mainwindow.h"
 #include "summaryview.h"
 #include "log.h"
+#include "tags.h"
 #include "timing.h"
 #include "inc.h"
 
@@ -809,7 +810,7 @@ parse_again:
 	g_slist_free(orig);
 	
 	for (cur = result; cur; cur = cur->next)
-		debug_print("sort after  %s\n", ((MsgInfo *)cur->data)->from);
+		debug_print("sort after %s\n", ((MsgInfo *)cur->data)->from);
 
 	debug_print("\n");
 
@@ -1079,7 +1080,7 @@ void procmsg_print_message(MsgInfo *msginfo, const gchar *cmdline)
 	FILE *tmpfp, *prfp;
 	gchar buf[1024];
 	gchar *p;
-
+	int r;
 	g_return_if_fail(msginfo);
 
 	if (procmime_msginfo_is_encrypted(msginfo))
@@ -1101,17 +1102,17 @@ void procmsg_print_message(MsgInfo *msginfo, const gchar *cmdline)
 		return;
 	}
 
-	if (msginfo->date) fprintf(prfp, "Date: %s\n", msginfo->date);
-	if (msginfo->from) fprintf(prfp, "From: %s\n", msginfo->from);
-	if (msginfo->to)   fprintf(prfp, "To: %s\n", msginfo->to);
-	if (msginfo->cc)   fprintf(prfp, "Cc: %s\n", msginfo->cc);
+	if (msginfo->date) r = fprintf(prfp, "Date: %s\n", msginfo->date);
+	if (msginfo->from) r = fprintf(prfp, "From: %s\n", msginfo->from);
+	if (msginfo->to)   r = fprintf(prfp, "To: %s\n", msginfo->to);
+	if (msginfo->cc)   r = fprintf(prfp, "Cc: %s\n", msginfo->cc);
 	if (msginfo->newsgroups)
-		fprintf(prfp, "Newsgroups: %s\n", msginfo->newsgroups);
-	if (msginfo->subject) fprintf(prfp, "Subject: %s\n", msginfo->subject);
+		r = fprintf(prfp, "Newsgroups: %s\n", msginfo->newsgroups);
+	if (msginfo->subject) r = fprintf(prfp, "Subject: %s\n", msginfo->subject);
 	fputc('\n', prfp);
 
 	while (fgets(buf, sizeof(buf), tmpfp) != NULL)
-		fputs(buf, prfp);
+		r = fputs(buf, prfp);
 
 	fclose(prfp);
 	fclose(tmpfp);
@@ -1217,25 +1218,18 @@ MsgInfo *procmsg_msginfo_copy(MsgInfo *msginfo)
 	return newmsginfo;
 }
 
-MsgInfo *procmsg_msginfo_get_full_info(MsgInfo *msginfo)
+MsgInfo *procmsg_msginfo_get_full_info_from_file(MsgInfo *msginfo, const gchar *file)
 {
 	MsgInfo *full_msginfo;
-	gchar *file;
 
 	if (msginfo == NULL) return NULL;
 
-	file = procmsg_get_message_file_path(msginfo);
 	if (!file || !is_file_exist(file)) {
-		g_free(file);
-		file = procmsg_get_message_file(msginfo);
-	}
-	if (!file || !is_file_exist(file)) {
-		g_warning("procmsg_msginfo_get_full_info(): can't get message file.\n");
+		g_warning("procmsg_msginfo_get_full_info_from_file(): can't get message file.\n");
 		return NULL;
 	}
 
 	full_msginfo = procheader_parse_file(file, msginfo->flags, TRUE, FALSE);
-	g_free(file);
 	if (!full_msginfo) return NULL;
 
 	msginfo->total_size = full_msginfo->total_size;
@@ -1281,6 +1275,28 @@ MsgInfo *procmsg_msginfo_get_full_info(MsgInfo *msginfo)
 	return procmsg_msginfo_new_ref(msginfo);
 }
 
+MsgInfo *procmsg_msginfo_get_full_info(MsgInfo *msginfo)
+{
+	MsgInfo *full_msginfo;
+	gchar *file;
+
+	if (msginfo == NULL) return NULL;
+
+	file = procmsg_get_message_file_path(msginfo);
+	if (!file || !is_file_exist(file)) {
+		g_free(file);
+		file = procmsg_get_message_file(msginfo);
+	}
+	if (!file || !is_file_exist(file)) {
+		g_warning("procmsg_msginfo_get_full_info(): can't get message file.\n");
+		return NULL;
+	}
+
+	full_msginfo = procmsg_msginfo_get_full_info_from_file(msginfo, file);
+	g_free(file);
+	return full_msginfo;
+}
+
 void procmsg_msginfo_free(MsgInfo *msginfo)
 {
 	if (msginfo == NULL) return;
@@ -1326,6 +1342,7 @@ void procmsg_msginfo_free(MsgInfo *msginfo)
 	}
 	slist_free_strings(msginfo->references);
 	g_slist_free(msginfo->references);
+	g_slist_free(msginfo->tags);
 
 	g_free(msginfo->plaintext_file);
 
@@ -1335,7 +1352,7 @@ void procmsg_msginfo_free(MsgInfo *msginfo)
 guint procmsg_msginfo_memusage(MsgInfo *msginfo)
 {
 	guint memusage = 0;
-	GSList *refs;
+	GSList *tmp;
 	
 	memusage += sizeof(MsgInfo);
 	if (msginfo->fromname)
@@ -1356,13 +1373,17 @@ guint procmsg_msginfo_memusage(MsgInfo *msginfo)
 		memusage += strlen(msginfo->msgid);
 	if (msginfo->inreplyto)
 		memusage += strlen(msginfo->inreplyto);
-	for (refs = msginfo->references; refs; refs=refs->next) {
-		gchar *r = (gchar *)refs->data;
-		memusage += r?strlen(r):0;
+
+	for (tmp = msginfo->references; tmp; tmp=tmp->next) {
+		gchar *r = (gchar *)tmp->data;
+		memusage += r?strlen(r):0 + sizeof(GSList);
 	}
 	if (msginfo->fromspace)
 		memusage += strlen(msginfo->fromspace);
 
+	for (tmp = msginfo->tags; tmp; tmp=tmp->next) {
+		memusage += sizeof(GSList);
+	}
 	if (msginfo->extradata) {
 		memusage += sizeof(MsgInfoExtraData);
 		if (msginfo->extradata->xface)
@@ -1632,7 +1653,7 @@ send_mail:
 		FILE *tmpfp;
 
     		/* write to temporary file */
-    		tmp = g_strdup_printf("%s%ctmp%p", g_get_tmp_dir(),
+    		tmp = g_strdup_printf("%s%cnntp%p", get_tmp_dir(),
                     	    G_DIR_SEPARATOR, file);
     		if ((tmpfp = g_fopen(tmp, "wb")) == NULL) {
             		FILE_OP_ERROR(tmp, "fopen");
@@ -1842,6 +1863,9 @@ void procmsg_msginfo_set_flags(MsgInfo *msginfo, MsgPermFlags perm_flags, MsgTmp
 	if ((perm_flags & MSG_IGNORE_THREAD) || (perm_flags_old & MSG_IGNORE_THREAD)) {
 		perm_flags_new &= ~(MSG_NEW | MSG_UNREAD);
 	}
+	if ((perm_flags & MSG_WATCH_THREAD) || (perm_flags_old & MSG_WATCH_THREAD)) {
+		perm_flags_new &= ~(MSG_IGNORE_THREAD);
+	}
 
 	if (perm_flags_old != perm_flags_new) {
 		folder_item_change_msg_flags(msginfo->folder, msginfo, perm_flags_new);
@@ -1920,6 +1944,9 @@ void procmsg_msginfo_change_flags(MsgInfo *msginfo,
 	if ((add_perm_flags & MSG_IGNORE_THREAD) || (perm_flags_old & MSG_IGNORE_THREAD)) {
 		perm_flags_new &= ~(MSG_NEW | MSG_UNREAD);
 	}
+	if ((add_perm_flags & MSG_WATCH_THREAD) || (perm_flags_old & MSG_WATCH_THREAD)) {
+		perm_flags_new &= ~(MSG_IGNORE_THREAD);
+	}
 
 	if (perm_flags_old != perm_flags_new) {
 		folder_item_change_msg_flags(msginfo->folder, msginfo, perm_flags_new);
@@ -2356,3 +2383,68 @@ gboolean procmsg_have_trashed_mails_fast (void)
 	folder_func_to_all_folders(item_has_trashed_mails, &result);
 	return result;
 }
+
+gchar *procmsg_msginfo_get_tags_str(MsgInfo *msginfo)
+{
+	GSList *cur = NULL;
+	gchar *tags = NULL;
+	
+	if (!msginfo)
+		return NULL;
+
+	if (msginfo->tags == NULL)
+		return NULL;
+	for (cur = msginfo->tags; cur; cur = cur->next) {
+		const gchar *tag = tags_get_tag(GPOINTER_TO_INT(cur->data));
+		if (!tag)
+			continue;
+		if (!tags)
+			tags = g_strdup(tag);
+		else {
+			int olen = strlen(tags);
+			int nlen = olen + strlen(tag) + 2 /* strlen(", ") */;
+			tags = g_realloc(tags, nlen+1);
+			if (!tags)
+				return NULL;
+			strcpy(tags+olen, ", ");
+			strcpy(tags+olen+2, tag);
+			tags[nlen]='\0';
+		}
+	}
+	return tags;
+}
+
+void procmsg_msginfo_update_tags(MsgInfo *msginfo, gboolean set, gint id)
+{
+	GSList changed;
+
+	if (id == 0)
+		return;
+
+	if (!set) {
+		msginfo->tags = g_slist_remove(
+					msginfo->tags,
+					GINT_TO_POINTER(id));
+		changed.data = GINT_TO_POINTER(id);
+		changed.next = NULL;
+		folder_item_commit_tags(msginfo->folder, msginfo, NULL, &changed);
+	} else {
+		if (!g_slist_find(msginfo->tags, GINT_TO_POINTER(id))) {
+			msginfo->tags = g_slist_append(
+					msginfo->tags,
+					GINT_TO_POINTER(id));
+		}
+		changed.data = GINT_TO_POINTER(id);
+		changed.next = NULL;
+		folder_item_commit_tags(msginfo->folder, msginfo, &changed, NULL);
+	}
+	
+}
+
+void procmsg_msginfo_clear_tags(MsgInfo *msginfo)
+{
+	GSList *unset = msginfo->tags;
+	msginfo->tags = NULL;
+	folder_item_commit_tags(msginfo->folder, msginfo, NULL, unset);
+	g_slist_free(unset);
+}
