@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifndef __SOURCEWINDOW_H__
@@ -39,10 +39,7 @@ struct _SourceWindow
 
 SourceWindow *source_window_create	(void);
 void source_window_show			(SourceWindow	*sourcewin);
-void source_window_destroy		(SourceWindow	*sourcewin);
 void source_window_show_msg		(SourceWindow	*sourcewin,
 					 MsgInfo	*msginfo);
-void source_window_append		(SourceWindow	*sourcewin,
-					 const gchar	*str);
 
 #endif /* __SOURCEWINDOW_H__ */
