@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "defs.h"
@@ -32,6 +32,7 @@
 #include "pixmaps/addr_one.xpm"
 #include "pixmaps/addr_two.xpm"
 #include "pixmaps/address.xpm"
+#include "pixmaps/anonymous.xpm"
 #include "pixmaps/book.xpm"
 #include "pixmaps/category.xpm"
 #include "pixmaps/checkbox_off.xpm"
@@ -45,6 +46,7 @@
 #include "pixmaps/edit_extern.xpm"
 #include "pixmaps/forwarded.xpm"
 #include "pixmaps/group.xpm"
+#include "pixmaps/insert_file.xpm"
 #include "pixmaps/interface.xpm"
 #include "pixmaps/jpilot.xpm"
 #include "pixmaps/key.xpm"
@@ -81,7 +83,11 @@
 #include "pixmaps/queue_open.xpm"
 #include "pixmaps/queue_open_hrm.xpm"
 #include "pixmaps/claws-mail_icon.xpm"
+#ifndef MAEMO
 #include "pixmaps/claws-mail_logo.xpm"
+#else
+#include "pixmaps/claws-mail_logo-small.xpm"
+#endif
 #include "pixmaps/address_book.xpm"
 #include "pixmaps/unread.xpm"
 #include "pixmaps/read.xpm"
@@ -95,6 +101,7 @@
 #include "pixmaps/quicksearch.xpm"
 #include "pixmaps/clip_gpg_signed.xpm"
 #include "pixmaps/gpg_signed.xpm"
+#include "pixmaps/go_folders.xpm"
 #include "pixmaps/mime_text_plain.xpm"
 #include "pixmaps/mime_text_html.xpm"
 #include "pixmaps/mime_application.xpm"
@@ -166,7 +173,18 @@
 #include "pixmaps/moved.xpm"
 #include "pixmaps/copied.xpm"
 #include "pixmaps/selection.xpm"
+#include "pixmaps/watchthread.xpm"
 #include "pixmaps/empty.xpm"
+#include "pixmaps/tray_newmail.offline.xpm"
+#include "pixmaps/tray_newmail.xpm"
+#include "pixmaps/tray_newmarkedmail.offline.xpm"
+#include "pixmaps/tray_newmarkedmail.xpm"
+#include "pixmaps/tray_nomail.offline.xpm"
+#include "pixmaps/tray_nomail.xpm"
+#include "pixmaps/tray_unreadmail.offline.xpm"
+#include "pixmaps/tray_unreadmail.xpm"
+#include "pixmaps/tray_unreadmarkedmail.offline.xpm"
+#include "pixmaps/tray_unreadmarkedmail.xpm"
 
 typedef struct _StockPixmapData	StockPixmapData;
 
@@ -195,6 +213,7 @@ struct _OverlayData
 	OverlayPosition position;
 	gint border_x;
 	gint border_y;
+	gboolean highlight;
 };
 
 static void stock_pixmap_find_themes_in_dir(GList **list, const gchar *dirname);
@@ -206,6 +225,7 @@ static StockPixmapData pixmaps[] =
 	{address_xpm				, NULL, NULL, "address", NULL},
 	{address_book_xpm			, NULL, NULL, "address_book", NULL},
 	{address_search_xpm			, NULL, NULL, "address_search", NULL},
+	{anonymous_xpm				, NULL, NULL, "anonymous", NULL},
 	{book_xpm				, NULL, NULL, "book", NULL},
 	{category_xpm				, NULL, NULL, "category", NULL},
 	{checkbox_off_xpm			, NULL, NULL, "checkbox_off", NULL},
@@ -242,7 +262,7 @@ static StockPixmapData pixmaps[] =
 	{inbox_close_hrm_mark_xpm		, NULL, NULL, "inbox_close_hrm_mark", NULL},
 	{inbox_open_mark_xpm			, NULL, NULL, "inbox_open_mark", NULL},
 	{inbox_open_hrm_mark_xpm		, NULL, NULL, "inbox_open_hrm_mark", NULL},
-	{paste_xpm				, NULL, NULL, "insert_file", NULL},
+	{insert_file_xpm			, NULL, NULL, "insert_file", NULL},
 	{interface_xpm				, NULL, NULL, "interface", NULL},
 	{jpilot_xpm				, NULL, NULL, "jpilot", NULL},
 	{key_xpm				, NULL, NULL, "key", NULL},
@@ -303,6 +323,7 @@ static StockPixmapData pixmaps[] =
 	{notice_note_xpm			, NULL, NULL, "notice_note",  NULL},
 	{quicksearch_xpm			, NULL, NULL, "quicksearch",  NULL},
 	{gpg_signed_xpm				, NULL, NULL, "gpg_signed", NULL},
+	{go_folders_xpm				, NULL, NULL, "go_folders", NULL},
 	{drafts_close_xpm			, NULL, NULL, "drafts_close", NULL},
 	{drafts_open_xpm			, NULL, NULL, "drafts_open", NULL},
 	{drafts_close_mark_xpm			, NULL, NULL, "drafts_close_mark", NULL},
@@ -334,7 +355,11 @@ static StockPixmapData pixmaps[] =
 	{claws_mail_icon_xpm			, NULL, NULL, "claws_mail_icon", NULL},
 	{read_xpm				, NULL, NULL, "read", NULL},
 	{trash_btn_xpm				, NULL, NULL, "trash_btn", NULL},
+#ifndef MAEMO
 	{claws_mail_logo_xpm			, NULL, NULL, "claws_mail_logo", NULL},
+#else
+	{claws_mail_logo_small_xpm		, NULL, NULL, "claws_mail_logo_small", NULL},
+#endif
         {dir_noselect_xpm                       , NULL, NULL, "dir_noselect" , NULL},
         {spam_xpm                               , NULL, NULL, "spam" , NULL},
         {spam_btn_xpm                           , NULL, NULL, "spam_btn" , NULL},
@@ -342,6 +367,17 @@ static StockPixmapData pixmaps[] =
 	{moved_xpm				, NULL, NULL, "moved", NULL},
 	{copied_xpm				, NULL, NULL, "copied", NULL},
 	{selection_xpm				, NULL, NULL, "selection", NULL},
+	{watchthread_xpm			, NULL, NULL, "watchthread", NULL},
+	{tray_newmail_offline_xpm		, NULL, NULL, "tray_newmail_offline", NULL},
+	{tray_newmail_xpm			, NULL, NULL, "tray_newmail", NULL},
+	{tray_newmarkedmail_offline_xpm		, NULL, NULL, "tray_newmarkedmail_offline", NULL},
+	{tray_newmarkedmail_xpm			, NULL, NULL, "tray_newmarkedmail", NULL},
+	{tray_nomail_offline_xpm		, NULL, NULL, "tray_nomail_offline", NULL},
+	{tray_nomail_xpm			, NULL, NULL, "tray_nomail", NULL},
+	{tray_unreadmail_offline_xpm		, NULL, NULL, "tray_unreadmail_offline", NULL},
+	{tray_unreadmail_xpm			, NULL, NULL, "tray_unreadmail", NULL},
+	{tray_unreadmarkedmail_offline_xpm	, NULL, NULL, "tray_unreadmarkedmail_offline", NULL},
+	{tray_unreadmarkedmail_xpm		, NULL, NULL, "tray_unreadmarkedmail", NULL},
         {empty_xpm                              , NULL, NULL, "empty" , NULL}
 };
 
@@ -489,7 +525,7 @@ static void stock_pixmap_find_themes_in_dir(GList **list, const gchar *dirname)
 	DIR *dp;
 	
 	if ((dp = opendir(dirname)) == NULL) {
-		debug_print("dir %s not found, skipping theme scan", dirname);
+		debug_print("dir %s not found, skipping theme scan", dirname?dirname:"(null)");
 		return;
 	}
 	
@@ -529,8 +565,7 @@ GList *stock_pixmap_themes_list_new(void)
 	GList *list = NULL;
 	
 	defaulttheme = g_strdup(DEFAULT_PIXMAP_THEME);
-	userthemes   = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S,
-				   RC_DIR, G_DIR_SEPARATOR_S, 
+	userthemes   = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, 
 				   PIXMAP_THEME_DIR, NULL);
 	systemthemes = g_strconcat(PACKAGE_DATA_DIR, G_DIR_SEPARATOR_S,
 				   PIXMAP_THEME_DIR, NULL);
@@ -589,6 +624,23 @@ static gboolean pixmap_with_overlay_expose_event_cb(GtkWidget *widget, GdkEventE
 	gdk_window_clear_area (drawable, expose->area.x, expose->area.y,
 			       expose->area.width, expose->area.height);
 
+	if (data->highlight) {
+		MainWindow *mw = NULL;
+
+		mw = mainwindow_get_mainwindow();
+		if (mw != NULL && mw->menubar != NULL) {
+			GdkGC *gc_frame = gdk_gc_new((GdkWindow *)drawable);
+			GdkColor color = gtk_widget_get_style(mw->menubar)->base[GTK_STATE_SELECTED];
+
+			gdk_gc_set_foreground(gc_frame, &color);
+			gdk_gc_set_line_attributes(gc_frame, 1, GDK_LINE_SOLID,
+					GDK_CAP_BUTT, GDK_JOIN_BEVEL);
+			gdk_draw_rectangle(drawable, gc_frame, FALSE, data->border_x-2, data->border_y-2, 
+					data->base_width+3, data->base_height+3);
+			g_object_unref(gc_frame);
+		}
+	}
+
 	gdk_gc_set_tile(gc_pix, data->base_pixmap);
 	gdk_gc_set_ts_origin(gc_pix, data->border_x, data->border_y);
 	gdk_gc_set_clip_mask(gc_pix, data->base_mask);
@@ -726,6 +778,7 @@ GtkWidget *stock_pixmap_widget_with_overlay(GtkWidget *window, StockPixmap icon,
 	data->position = pos;
 	data->border_x = border_x;
 	data->border_y = border_y;
+	data->highlight = FALSE;
 
 	widget = gtk_drawing_area_new();
 	gtk_drawing_area_size(GTK_DRAWING_AREA(widget), data->base_width + border_x * 2, 
@@ -734,6 +787,7 @@ GtkWidget *stock_pixmap_widget_with_overlay(GtkWidget *window, StockPixmap icon,
 			 G_CALLBACK(pixmap_with_overlay_expose_event_cb), data);
 	g_signal_connect(G_OBJECT(widget), "destroy",
 			 G_CALLBACK(pixmap_with_overlay_destroy_cb), data);
+	g_object_set_data(G_OBJECT(widget), "highlight", &(data->highlight));
 	return widget;
 
 }
